package stocks.test;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import org.omg.CORBA.ORB;

import stocks.*;
import stocks.StockInfoPackage.StockType;
import stocks.client.*;
import junit.framework.TestCase;


public class StockQuoterTest extends TestCase {
	QuoterFactory _serverFactory = null;
	ClientQuoterFactory _clientFactory = null;
  ORB _orb = null;

  public static void main(String[] args) {
    junit.textui.TestRunner.run(StockQuoterTest.class);
  }

  public StockQuoterTest(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();

    String[] args = null;
    _orb = ORB.init(args,null);
    
    _clientFactory = new ClientQuoterFactoryImpl(_orb);

    String filename = "quoter_factory.ior";
    FileInputStream fis = new FileInputStream(filename);
    BufferedReader d = new BufferedReader(new InputStreamReader(fis));
    String ior = d.readLine();
    
    org.omg.CORBA.Object obj = _orb.string_to_object(ior);
    _serverFactory = QuoterFactoryHelper.narrow(obj);
  }

  protected void tearDown() throws Exception {
    super.tearDown();
    
    _clientFactory = null;
    _serverFactory = null;
    _orb = null;
  }

  /** Server test.
   * Create a quoter and query for a certain quote.
   * Must go well.
   */
  public void test1() throws Exception {
    String name = "Dow Jones";
    Quoter dowjones = _serverFactory.create_quoter(name);
    assertEquals( name, dowjones.quoter_name() );
    
    int iQuote = dowjones.get_quote("Boeing");
    assertEquals( 67, iQuote );
  }
  
  /** Server test.
   * Test inheritance between quoter interfaces.
   * Must go well.
   */
  public void test2() throws Exception {
    Quoter obj = _serverFactory.create_quoter("Dow Jones");
    StockQuoter dowjones = StockQuoterHelper.narrow(obj);
  }

  /** Server test.
   * Query all properties of a certain stock.
   * Must go well.
   */
  public void test3() throws Exception {
    Quoter obj = _serverFactory.create_quoter("Dow Jones");
    StockQuoter dowjones = StockQuoterHelper.narrow(obj);
    
    String name = "Intel";
    StockInfo intel = dowjones.get_stock_info(name);
    assertEquals( name, intel.name );
    assertEquals( "INTC", intel.symbol );
    assertEquals( "US4581401001", intel.ISIN );
    assertEquals( StockType._tresury_stock, intel.type.value() );
    assertEquals( 29, intel.quote.high );
    assertEquals( 27, intel.quote.low );
    assertEquals( 27, intel.quote.last );
  }

  /** Server test.
   * Try to create a quoter with invalid name.
   * Must fail with a InvalidQuoter exception.
   */
  public void test4() throws Exception {
    try {
      Quoter obj = _serverFactory.create_quoter("");
      fail("Created quoter with invalid name.");
    }
    catch( InvalidQuoter e ) {
      // pass...
    }
  }

  /** Server test.
   * Try to query a non-existing stock on both quoter interfaces.
   * Must fail with a InvalidStock exception both times.
   */
  public void test5() throws Exception {
    Quoter quoter1 = _serverFactory.create_quoter("Dow Jones");
    try {
      quoter1.get_quote("");
      fail("Queried quote for a non-existing stock");
    }
    catch( InvalidStock e ) {
      // pass...
    }
    
    StockQuoter quoter2 = StockQuoterHelper.narrow(quoter1);
    try {
      quoter2.get_stock_info("");
      fail("Queried quote for a non-existing stock");
    }
    catch( InvalidStock e ) {
      // pass...
    }
  }

  /** Client test.
   * Create a quoter and query for a certain quote.
   */
  public void test6() throws Exception {
    String name = "Dow Jones";
    ClientStockQuoter dowjones = _clientFactory.create_quoter(name);
    assertEquals( name, dowjones.getQuoterName() );
    
    ClientStock boeing = dowjones.getStock("Boeing");
    assertEquals( 67, boeing.getLast() );
  }

  /** Client test.
   * 
   */
  public void test7() throws Exception {
    ClientStockQuoter dowjones = _clientFactory.create_quoter("Dow Jones");

    String name = "Intel";
    ClientStock intel = dowjones.getStock(name);
    assertEquals( name, intel.getName() );
    assertEquals( "INTC", intel.getSymbol() );
    assertEquals( "US4581401001", intel.getISIN() );
    assertEquals( ClientStock.tresury_stock, intel.getStockType() );
    assertEquals( 29, intel.getHigh() );
    assertEquals( 27, intel.getLow() );
    assertEquals( 27, intel.getLast() );
  }

  /** Client test.
   * 
   */
  public void test8() throws Exception {
    ClientStockQuoter dowjones = _clientFactory.create_quoter("Dow Jones");

    String name = "Boeing";
    ClientStock intel = dowjones.getStock(name);
    assertEquals( name, intel.getName() );
    assertEquals( "BA", intel.getSymbol() );
    assertEquals( "US0970231058", intel.getISIN() );
    assertEquals( ClientStock.common_stock, intel.getStockType() );
    assertEquals( 68, intel.getHigh() );
    assertEquals( 64, intel.getLow() );
    assertEquals( 67, intel.getLast() );
  }

  /** Client test.
   * Try to create a quoter with invalid name.
   * Must fail with a InvalidQuoterException.
   */
  public void test9() throws Exception {
    try {
      ClientStockQuoter quoter = _clientFactory.create_quoter("");
      fail("Created quoter with invalid name.");
    }
    catch( InvalidQuoterException e ) {
      // pass...
    }
  }

  /** Client test.
   * Try to query a non-existing stock.
   * Must fail with a InvalidStockException.
   */
  public void test10() throws Exception {
    ClientStockQuoter quoter = _clientFactory.create_quoter("Dow Jones");
    try {
      quoter.getStock("");
      fail("Queried quote for a non-existing stock");
    }
    catch( InvalidStockException e ) {
      // Pass...
    }
  }
}
